#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <shellapi.h>
#include "AkelDLL.h"


//Functions prototypes
LRESULT CALLBACK NewEditParentProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
void Hide();
void Show();
void InitCommon(PLUGINDATA *pd);
void InitMain();
void UninitMain();

//Global variables
HWND hMainWnd;
BOOL bOldWindows;
BOOL bMDI;
HICON hMainIcon;
HICON hMainSmallIcon=NULL;
BOOL bInitCommon=FALSE;
BOOL bInitMain=FALSE;
WNDPROCDATA *NewMainProcData=NULL;
WNDPROCDATA *NewFrameProcData=NULL;
UINT nMinimizeToTrayMessage=0;
DWORD dwMainStyle;
DWORD dwCmdShow;
BOOL bMinimizeToTrayAlways=FALSE;
BOOL bMinimizeToTrayNowOnStart=FALSE;
BOOL bCmdShow=FALSE;


//Identification
extern "C" void __declspec(dllexport) DllAkelPadID(PLUGINVERSION *pv)
{
  pv->dwAkelDllVersion=AKELDLL;
  pv->dwExeMinVersion3x=MAKE_IDENTIFIER(3, 7, 4, 0);
  pv->dwExeMinVersion4x=MAKE_IDENTIFIER(4, 4, 0, 0);
  pv->pPluginName="MinimizeToTray";
}

//Plugin extern function
extern "C" void __declspec(dllexport) Always(PLUGINDATA *pd)
{
  if (pd->lpbAutoLoad)
  {
    *pd->lpbAutoLoad=TRUE;
    return;
  }

  if (!bInitCommon) InitCommon(pd);
  if (!bInitMain) InitMain();

  if (bMinimizeToTrayAlways)
  {
    bMinimizeToTrayAlways=FALSE;

    //Stay in memory, but show as non-active
    pd->nUnload=UD_NONUNLOAD_NONACTIVE;
  }
  else
  {
    bMinimizeToTrayAlways=TRUE;

    //Stay in memory, and show as active
    pd->nUnload=UD_NONUNLOAD_ACTIVE;
  }
}

extern "C" void __declspec(dllexport) Now(PLUGINDATA *pd)
{
  if (pd->lpbAutoLoad)
  {
    *pd->lpbAutoLoad=TRUE;
    return;
  }

  if (pd->bOnStart) bMinimizeToTrayNowOnStart=TRUE;
  if (!bInitCommon) InitCommon(pd);
  if (!bInitMain) InitMain();

  Hide();

  //Stay in memory, but show as non-active
  pd->nUnload=UD_NONUNLOAD_NONACTIVE;
}

LRESULT CALLBACK NewEditParentProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  if (uMsg == AKDN_MAIN_ONSTART_PRESHOW)
  {
    if (bMinimizeToTrayNowOnStart)
    {
      NMAINSHOW *nms=(NMAINSHOW *)lParam;

      dwMainStyle=*nms->dwStyle;
      dwCmdShow=*nms->dwShow;
      nms->bProcess=FALSE;
      bCmdShow=TRUE;
    }
  }
  else if (uMsg == AKDN_MAIN_ONSTART_FINISH)
  {
    bMinimizeToTrayNowOnStart=FALSE;
  }
  else if (uMsg == WM_SYSCOMMAND)
  {
    if (hWnd == hMainWnd)
    {
      if (wParam == SC_MINIMIZE)
      {
        if (bMinimizeToTrayAlways)
        {
          Hide();
          return TRUE;
        }
      }
    }
  }
  else if (uMsg == nMinimizeToTrayMessage)
  {
    if (lParam == WM_LBUTTONUP ||
        lParam == WM_RBUTTONUP)
    {
      Show();
      return TRUE;
    }
  }
  else if (uMsg == AKDN_ACTIVATE)
  {
    if (!bMinimizeToTrayNowOnStart)
    {
      if (!IsWindowVisible(hMainWnd))
        Show();
    }
  }

  //Call next procedure
  if (hWnd == hMainWnd)
    return NewMainProcData->NextProc(hWnd, uMsg, wParam, lParam);
  else
    return NewFrameProcData->NextProc(hWnd, uMsg, wParam, lParam);
}

void Hide()
{
  NOTIFYICONDATA nid;
  HWND hWndNext;

  //Add notify icon
  nid.cbSize=sizeof(NOTIFYICONDATA);
  nid.hWnd=hMainWnd;
  nid.uID=0;
  nid.uFlags=NIF_ICON|NIF_MESSAGE;
  nid.uCallbackMessage=nMinimizeToTrayMessage;
  nid.hIcon=(HICON)hMainSmallIcon;
  Shell_NotifyIcon(NIM_ADD, &nid);

  if (!bCmdShow)
  {
    //Activate next window
    if (hWndNext=GetNextWindow(hMainWnd, GW_HWNDNEXT))
      BringWindowToTop(hWndNext);

    //Hide main window
    ShowWindow(hMainWnd, SW_HIDE);
  }
}

void Show()
{
  NOTIFYICONDATA nid;

  //Delete notify icon
  nid.cbSize=sizeof(NOTIFYICONDATA);
  nid.hWnd=hMainWnd;
  nid.uID=0;
  Shell_NotifyIcon(NIM_DELETE, &nid);

  //Show main window
  if (bCmdShow)
  {
    ShowWindow(hMainWnd, (dwMainStyle == WS_MAXIMIZE)?SW_SHOWMAXIMIZED:SW_SHOW);
    if (dwCmdShow != SW_SHOWNORMAL) ShowWindow(hMainWnd, dwCmdShow);
    SetForegroundWindow(hMainWnd);
    bCmdShow=FALSE;
  }
  else
  {
    ShowWindow(hMainWnd, SW_SHOW);
    BringWindowToTop(hMainWnd);
  }
}

void InitCommon(PLUGINDATA *pd)
{
  bInitCommon=TRUE;
  hMainWnd=pd->hMainWnd;
  bOldWindows=pd->bOldWindows;
  bMDI=pd->bMDI;
  hMainIcon=pd->hMainIcon;
}

void InitMain()
{
  bInitMain=TRUE;

  //Get 16x16 icon
  hMainSmallIcon=(HICON)CopyImage((HANDLE)hMainIcon, IMAGE_ICON, GetSystemMetrics(SM_CXSMICON), GetSystemMetrics(SM_CYSMICON), LR_COPYFROMRESOURCE);

  //Register unique message
  if (!nMinimizeToTrayMessage)
    nMinimizeToTrayMessage=RegisterWindowMessageA("MinimizeToTray");

  //SubClass
  NewMainProcData=NULL;
  SendMessage(hMainWnd, AKD_SETMAINPROC, (WPARAM)NewEditParentProc, (LPARAM)&NewMainProcData);

  if (bMDI)
  {
    NewFrameProcData=NULL;
    SendMessage(hMainWnd, AKD_SETFRAMEPROC, (WPARAM)NewEditParentProc, (LPARAM)&NewFrameProcData);
  }
}

void UninitMain()
{
  bInitMain=FALSE;

  //Destroy 16x16 icon
  if (hMainSmallIcon)
  {
    DestroyIcon(hMainSmallIcon);
    hMainSmallIcon=NULL;
  }

  //Remove subclass
  if (NewMainProcData)
  {
    SendMessage(hMainWnd, AKD_SETMAINPROC, (WPARAM)NULL, (LPARAM)&NewMainProcData);
    NewMainProcData=NULL;
  }
  if (bMDI)
  {
    if (NewFrameProcData)
    {
      SendMessage(hMainWnd, AKD_SETFRAMEPROC, (WPARAM)NULL, (LPARAM)&NewFrameProcData);
      NewFrameProcData=NULL;
    }
  }
}

//Entry point
extern "C" BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
  if (fdwReason == DLL_PROCESS_ATTACH)
  {
  }
  else if (fdwReason == DLL_THREAD_ATTACH)
  {
  }
  else if (fdwReason == DLL_THREAD_DETACH)
  {
  }
  else if (fdwReason == DLL_PROCESS_DETACH)
  {
    if (bInitMain) UninitMain();
  }
  return TRUE;
}
